/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.grammar;

/**
 * This exception should a grammar produce, if it isn't valid
 *
 * @author Stephan Michels
 * @version %I%, %G%
 */
public class SyntaxErrorException extends Exception
{
  private String publicId = "";
  private String systemId = "";
  private String message = "";

  /**
   * Create a syntax error exception
   *
   * @param message Message of the error
   */
  public SyntaxErrorException(String message)
  {
    this.message = message;
  }

  /**
   * Create a syntax error exception
   *
   * @param message Message of the error
   * @param publicId Public id of the grammar
   * @param systemId System id of the grammar
   */
  public SyntaxErrorException(String message, String publicId,
                              String systemId)
  {
    this.message = message;
    this.publicId = publicId;
    this.systemId = systemId;
  }

  /**
   * Returns the public id of the grammar
   *
   * @return Public id
   */
  public String getPublicId()
  {
    return publicId;
  }

  /**
   * Returns the system id of the grammar
   *
   * @return System id
   */
  public String getSystemId()
  {
    return systemId;
  }

  /**
   * Returns the message of the error
   *
   * @return Message of the error
   */
  public String getMessage()
  {
    return message;
  }

  /**
   * Convert the error to a string
   *
   * @return Content of the error
   */
  public String toString()
  {
    return message;
  }
}
